C-UL
C-UL   rmfrowinit: Initialization routine - must be called prior to
C-UL               calling routine rmfrow() below; simply places all
C-UL               its arguments into a common block such that they can
C-UL               be accessed within rmfrow
C-UL
C-RDS   v3.0 - changes from SSE update 29-Apr-2001

      subroutine rmfrowinit(epar_com, 
     *                      param_com, model_com,
     *                      grade_com, nepts_com, ewidth_com,
     *                      nebin_com,
     *                      threshold_com, back_sigma_com,
     *                      echlo_com, echhi_com,
     *                      si_kedge_com, si_kalpha_com)

      implicit none
      include 'rmfcommon.f'

      integer nepts_com(NEMAX), grade_com(NALL)
      integer nebin_com, model_com(NALL)
      real    epar_com(NEMAX), param_com(NPARAM, NEMAX, NALL)
      real    ewidth_com, threshold_com, back_sigma_com
      real    echlo_com(MAXCHAN), echhi_com(MAXCHAN)
      real    si_kedge_com, si_kalpha_com

      integer i, j, k

      jmax       = 1
      gmax       = 1
      nebin      = nebin_com
      ewidth     = ewidth_com
      threshold  = threshold_com
      back_sigma = back_sigma_com

      do i=1, MAXCHAN
          echlo(i) = echlo_com(i)
          echhi(i) = echhi_com(i)
      enddo

      do i=1, NALL
          model(i) = model_com(i)
          grade(i) = grade_com(i)
          nepts(i) = nepts_com(i)
      enddo

      do i=1, NEMAX
          epar(i) = epar_com(i)
      enddo

      do i=1, NPARAM
          do j=1, NEMAX
              do k=1, NALL
                  param(i, j, k) = param_com(i, j, k)
              enddo
          enddo
      enddo

      si_kedge = si_kedge_com
      si_kalpha = si_kalpha_com

      end


C-UL
C-UL  rmfrow : compute response for a single input energy
C-UL
C-UL  ARGUMENTS:
C-UL   IN:
C-UL     energy : input energy [eV]
C-UL     siabsd : Si absorption length [um]
C-UL     nch    : number of channels of output spectrum
C-UL     pattid: pattern ID
C-UL     pattfrac: pattern fraction vectors
C-UL   OUT:
C-UL     responserow : output spectrum of size nch
C-UL
      subroutine rmfrow(energy, siabsd, nch, pattid, pattfracchan,
     *                  pattfrac, responserow)

      implicit none

      include 'rmfcommon.f'

      integer nch, pattid, pattfracchan
      real energy, siabsd
      real responserow(nch), pattfrac(pattfracchan, *)


      real rmfgrade(MAXCHAN, MAXGRADE)
      real main_norm     
      real main_energy   
      real main_sigma    
      real sloss_alpha   
      real sloss_beta    
      real red_sigma
      real shelf_norm
      real lowe_norm
      real lowe_sigma
      real flo_norm      
      real flo_sigma     
      real esc_norm      
      real esc_sigma

      real escenergy
      real ytotal, x1, x2, xbin, xbin2
      real renorm(MAXGRADE)
      real gauss

      integer k, l, l1, l2, m, m1, elo, ehi
      integer jgmin,jgmax,jemin,jemax,jg,je
      real yrmf(10000), yconv(10000)
      real yold, ypnorm, abit, ctotal, rmfnorm    
      real event_threshold

      integer event_min(4), event_max(4)
      data event_min/1, 3, 5, 6/
      data event_max/2, 4, 5, 6/

      real ethresh(4)
      data ethresh/1.0, 2.0, 3.0, 4.0/

      integer idel
c
c Convert the pattern id into the grades which are wanted
c 0=singles, 1=dubs, 2=trips, 3=quads, 4=sings+dubs, 5=s+d+t+q
      if (pattid .lt. 4) then
         jgmin=pattid+1
         jgmax=pattid+1
      elseif (pattid .eq. 4) then
         jgmin=1
         jgmax=2
      elseif (pattid .eq. 5) then
         jgmin=1
         jgmax=4
      endif

c Zero rmfgrade
      do jg = jgmin, jgmax
        do k = 1, nch
          rmfgrade(k,jg) = 0.0
        enddo
      enddo

C-UL
C-UL   zero responserow
C-UL
      do k=1, nch
         responserow(k) = 0.
      enddo

c
c Loop over each pattern grade, and event type (i.e. main or leak). There
c are 4 (Grade 1-4) MAIN event types and 2 (Grade 1-2) LEAK type events.
c
      do jg = jgmin, jgmax

        event_threshold = threshold * ethresh(jg)
        jemin = event_min(jg)
        jemax = event_max(jg)

        do je = jemin, jemax

c
c Set up the parameters depending on whether you are above or below the
c si_kedge. 
c
c pcalc calculates the value of each parameter from the coefficient 
c arrays. pcalc also accepts lower and upper limits such that if the
c calculated value exceeds these limits, the parameter is set to the limit.
c
c If above the Si edge, the flouresence and escape peak sigmas are
c derived from the coefficient array of the main peak sigma.
c
c response calculates the rmf using the appropriate subset of parameters 
c for the given functional model, given the value of model. Not very
c efficient coding.
c
        call pinterp(energy,epar,nepts(je),param,1,je,0.0,main_norm)

        if (main_norm .gt. 0.0) then

        call pinterp(energy,epar,nepts(je),param,2,je,0.0,main_energy)
        call pinterp(energy,epar,nepts(je),param,3,je,5.0,main_sigma)
        call pinterp(energy,epar,nepts(je),param,4,je,0.0,sloss_alpha)
        call pinterp(energy,epar,nepts(je),param,5,je,0.0,sloss_beta)
        call pinterp(energy,epar,nepts(je),param,6,je,1.0,red_sigma)
        call pinterp(energy,epar,nepts(je),param,7,je,0.0,shelf_norm)
c          shelf_norm = shelf_norm * 0.85
        call pinterp(energy,epar,nepts(je),param,8,je,0.0,lowe_norm)
c          lowe_norm = lowe_norm * 0.85
        call pinterp(energy,epar,nepts(je),param,9,je,1.0,lowe_sigma)

        if (energy .ge. si_kedge) then
         call pinterp(energy,epar,nepts(je), param,10,je,0.0,flo_norm)
         call pinterp(si_kalpha,epar,nepts(je),param,3,je,5.0,flo_sigma)
         call pinterp(energy,epar,nepts(je), param,12,je,0.0,esc_norm)
         escenergy = energy - si_kalpha
         call pinterp(escenergy,epar,nepts(je),param,3,je,5.0,esc_sigma)
        endif

C        if (energy .gt. 1000. .and. energy .lt. 1010) then
C           write(*,*)(param(idel,24,1), param(idel,24,2), 
C     &                param(idel,24,3), param(idel,24,4), 
C     &                param(idel,24,5), param(idel,24,6), idel=1,13)
C        endif
c
c Calculate the surface loss lookup tables for the appropriate models
c
        if ((model(je) .eq. 0) .or. (model(je) .eq. 1) .or.
     &      (model(je) .eq. 2) .or. (model(je) .eq. 3) .or.
     &      (model(je) .eq. 6) .or. (model(je) .eq. 7) .or.
     &      (model(je) .eq. 8) .or. (model(je) .eq. 9)) then
            call d_lookup(siabsd, sloss_alpha, sloss_beta)
        endif
c
c Get the basic response at each native bin
c
          ytotal = 0.0
          xbin = 0.0
          do k = 1, nebin
            xbin = ewidth * float(k-1)
            if (xbin .ge. event_threshold) then
              call response(model(je), xbin, 
     &                    main_norm, main_energy, main_sigma,
     &                    red_sigma, shelf_norm, lowe_norm, lowe_sigma,
     &                    flo_norm, flo_sigma, esc_norm, esc_sigma, 
     &                    yrmf(k))

            else
              yrmf(k) = 0.0
            endif

            if (k .gt. 1) then
              ytotal = ytotal + (ewidth * (yrmf(k) + yold))
            endif
            yold = yrmf(k)

          enddo
          ytotal = 0.5 * ytotal

          if (ytotal .gt. 0.0) then

          if (back_sigma .gt. 0.0) then 
c
c With noise convolution

            ctotal = 0.0
            do k = 1, nebin

              xbin = ewidth * float(k-1)
              elo = 1 + int((xbin - (5.0 * back_sigma)) / ewidth)           
              ehi = 1 + int((xbin + (5.0 * back_sigma)) / ewidth)

              if (elo .lt. 1) then
                elo = 1
              endif

              if (ehi .gt. nebin) then
                ehi = nebin
              endif

              yconv(k) = 0.0
              do l = elo, ehi
                xbin2 = ewidth * float(l-1)
                yconv(k) = yconv(k) + 
     &                       (yrmf(l) * gauss(xbin2, xbin, back_sigma))
              enddo

              if (k .gt. 1) then
                ctotal = ctotal + (ewidth * (yconv(k) + yold))
              endif
              yold = yconv(k)

            enddo

            ctotal = ctotal * 0.5
            ypnorm = (ytotal / ctotal)
c
c Integrate the native bins over the data bins, adjusting the 
c integrated convolved rmf by the ratio of ytotal to ctotal
c
            do l = 1, nch

              l1 = 1 + int(echlo(l) / ewidth)
              l2 = 1 + int(echhi(l) / ewidth)

              do m = l1, l2
                x1 = ewidth * float(m-1)
                x2 = x1 + ewidth
                m1 = m + 1
                call getarea(x1, x2, yconv(m), yconv(m1), 
     &                       echlo(l), echhi(l), abit)

C                rmfgrade(l, jg) = rmfgrade(l, jg) + abit
                rmfgrade(l, jg) = rmfgrade(l, jg) + (abit*ypnorm)

              enddo
C              rmfgrade(l, jg) = rmfgrade(l, jg) * ypnorm

C            if (energy .gt. 1000. .and. energy .lt. 1010) then
C              write(*, *) echlo(l), rmfgrade(l,jg)
C            endif

            enddo

          else
c
c Without convolution
c
c Integrate the native bins over the data bins

            do l = 1, nch

              l1 = 1 + int(echlo(l) / ewidth)
              l2 = 1 + int(echhi(l) / ewidth)

              do m = l1, l2
                x1 = ewidth * float(m-1)
                x2 = x1 + ewidth
                m1 = m + 1
                
                call getarea(x1, x2, yrmf(m), yrmf(m1), 
     &                       echlo(l), echhi(l), abit) 
                rmfgrade(l, jg) = rmfgrade(l, jg) + abit
              enddo

            enddo

          endif

c ytotal check
        endif

c main-norm check
        endif

c loop over je
        enddo

c Loop over jg
        enddo
c
c Normalise each pattern grade, multiply by pattern fraction and add to rmf
c	
        do jg = jgmin, jgmax
          renorm(jg) = 0.0
          do k = 1, nch
            renorm(jg) = renorm(jg) + rmfgrade(k,jg)
          enddo
          if (renorm(jg) .gt. 0.0) then
            do k = 1, nch
              rmfgrade(k,jg) = rmfgrade(k,jg) / renorm(jg)
              responserow(k) = responserow(k) + 
     &                              pattfrac(k,jg)*rmfgrade(k,jg)
            enddo

          endif
        enddo

c Normalise rmf to 1.0
c
        rmfnorm = 0.0
        do k = 1, nch
          rmfnorm = rmfnorm + responserow(k)
        enddo
        if (rmfnorm .gt. 0.0) then
          do k = 1, nch
            responserow(k) =  responserow(k) / rmfnorm
          enddo
        endif

      return
      end


